#include "stm32f10x.h"
#include "max7219.h"
//----------------------------------------------------------
#define SYSCLOCK 72000000U
//----------------------------------------------------------
#define CS1_RESET() SET_BIT(GPIOA->ODR,GPIO_ODR_ODR4)
#define CS1_SET() CLEAR_BIT(GPIOA->ODR,GPIO_ODR_ODR4)
#define CS2_RESET() SET_BIT(GPIOB->ODR,GPIO_ODR_ODR12)
#define CS2_SET() CLEAR_BIT(GPIOB->ODR,GPIO_ODR_ODR12)
#define SPI1_ENABLE() SET_BIT(SPI1->CR1, SPI_CR1_SPE);
#define SPI2_ENABLE() SET_BIT(SPI2->CR1, SPI_CR1_SPE);
//----------------------------------------------------------
__IO uint32_t tmpreg;
__IO uint32_t SysTick_CNT = 0;
uint16_t src_buf[128] = {0};
uint16_t dst_buf[128] = {0};
uint8_t fl=0;
//----------------------------------------------------------
__forceinline void delay(__IO uint32_t tck)
{
  while(tck)
  {
    tck--;
  }  
}
//----------------------------------------------------------
void delay_ms(uint32_t ms)
{
  MODIFY_REG(SysTick->VAL,SysTick_VAL_CURRENT_Msk,SYSCLOCK / 1000 - 1);
  SysTick_CNT = ms;
  while(SysTick_CNT) {}
}
//-----------------------------------------------------------
void RCC_DeInit(void)
{
  SET_BIT(RCC->CR, RCC_CR_HSION);
  while(READ_BIT(RCC->CR, RCC_CR_HSIRDY == RESET)) {}
  MODIFY_REG(RCC->CR, RCC_CR_HSITRIM, 0x80U);
  CLEAR_REG(RCC->CFGR);
  while (READ_BIT(RCC->CFGR, RCC_CFGR_SWS) != RESET) {}
  CLEAR_BIT(RCC->CR, RCC_CR_PLLON);
  while (READ_BIT(RCC->CR, RCC_CR_PLLRDY) != RESET) {}
  CLEAR_BIT(RCC->CR, RCC_CR_HSEON | RCC_CR_CSSON);
  while (READ_BIT(RCC->CR, RCC_CR_HSERDY) != RESET) {}
  CLEAR_BIT(RCC->CR, RCC_CR_HSEBYP);
  //Reset all CSR flags
  SET_BIT(RCC->CSR, RCC_CSR_RMVF);
}
//----------------------------------------------------------
void SetSysClockTo72(void)
{
  SET_BIT(RCC->CR, RCC_CR_HSEON);
  while(READ_BIT(RCC->CR, RCC_CR_HSERDY == RESET)) {}
  //Enable the Prefetch Buffer
  CLEAR_BIT(FLASH->ACR, FLASH_ACR_PRFTBE);
  SET_BIT(FLASH->ACR, FLASH_ACR_PRFTBE);
  MODIFY_REG(FLASH->ACR, FLASH_ACR_LATENCY, FLASH_ACR_LATENCY_2);
  MODIFY_REG(RCC->CFGR, RCC_CFGR_HPRE, RCC_CFGR_HPRE_DIV1);
  MODIFY_REG(RCC->CFGR, RCC_CFGR_PPRE2, RCC_CFGR_PPRE2_DIV1);
  MODIFY_REG(RCC->CFGR, RCC_CFGR_PPRE1, RCC_CFGR_PPRE1_DIV2);
  MODIFY_REG(RCC->CFGR, RCC_CFGR_PLLSRC | RCC_CFGR_PLLXTPRE | RCC_CFGR_PLLMULL,
             RCC_CFGR_PLLSRC | RCC_CFGR_PLLMULL9);
  SET_BIT(RCC->CR, RCC_CR_PLLON);
  while(READ_BIT(RCC->CR, RCC_CR_PLLRDY) != (RCC_CR_PLLRDY)) {}
  MODIFY_REG(RCC->CFGR, RCC_CFGR_SW, RCC_CFGR_SW_PLL);
  while(READ_BIT(RCC->CFGR, RCC_CFGR_SWS) != RCC_CFGR_SWS_PLL) {}
}
//----------------------------------------------------------
void SysTick_Init(void)
{
  MODIFY_REG(SysTick->LOAD,SysTick_LOAD_RELOAD_Msk,SYSCLOCK / 1000 - 1);
  CLEAR_BIT(SysTick->VAL, SysTick_VAL_CURRENT_Msk);
  SET_BIT(SysTick->CTRL, SysTick_CTRL_CLKSOURCE_Msk | SysTick_CTRL_ENABLE_Msk | SysTick_CTRL_TICKINT_Msk);
}
//----------------------------------------------------------
static void GPIO_Init(void)
{
  SET_BIT(RCC->APB2ENR, RCC_APB2ENR_IOPBEN);
  MODIFY_REG(GPIOB->CRH,  GPIO_CRH_CNF12_1 | GPIO_CRH_CNF12_0 | GPIO_CRH_MODE12_1, GPIO_CRH_MODE12_0);
}
//----------------------------------------------------------
static void SPI1_Init(void)
{
  //SPI1 GPIO
  SET_BIT(RCC->APB2ENR, RCC_APB2ENR_IOPAEN);
  MODIFY_REG(GPIOA->CRL, GPIO_CRL_CNF4_1 | GPIO_CRL_CNF5_1 | GPIO_CRL_CNF6_0 | GPIO_CRL_CNF7_0 | GPIO_CRL_MODE7,
        GPIO_CRL_CNF4_0 | GPIO_CRL_CNF5_0 | GPIO_CRL_CNF6_1 | GPIO_CRL_CNF7_1 | \
        GPIO_CRL_MODE4 | GPIO_CRL_MODE5 | GPIO_CRL_MODE6);
  //RCC peripheral clock enabling
  SET_BIT(RCC->APB2ENR, RCC_APB2ENR_SPI1EN);
  //Delay after an RCC peripheral clock enabling
  tmpreg = READ_BIT(RCC->APB2ENR, RCC_APB2ENR_SPI1EN);
  //SPI1 DMA Init
  //SPI1_RX Init
  //Set transfer direction (Peripheral to Memory)
  CLEAR_BIT(DMA1_Channel2->CCR, DMA_CCR2_DIR | DMA_CCR2_MEM2MEM);
  //Set priority level
  CLEAR_BIT(DMA1_Channel2->CCR, DMA_CCR2_PL);
  //Transfer mode NORMAL
  CLEAR_BIT(DMA1_Channel2->CCR, DMA_CCR2_CIRC);
 //Set peripheral no increment mode
  CLEAR_BIT(DMA1_Channel2->CCR, DMA_CCR2_PINC);
  //Set memory increment mode
  SET_BIT(DMA1_Channel2->CCR, DMA_CCR2_MINC);
  //Set peripheral data width
  MODIFY_REG(DMA1_Channel2->CCR, DMA_CCR2_PSIZE_1, DMA_CCR2_PSIZE_0);
  //Set memory data width
  MODIFY_REG(DMA1_Channel2->CCR, DMA_CCR2_MSIZE_1, DMA_CCR2_MSIZE_0);
  //SPI1_TX Init
  //Set transfer direction (Memory to Peripheral)
  MODIFY_REG(DMA1_Channel3->CCR, DMA_CCR3_MEM2MEM, DMA_CCR3_DIR);
  //Set priority level
  CLEAR_BIT(DMA1_Channel3->CCR, DMA_CCR3_PL);
  //Transfer mode NORMAL
  CLEAR_BIT(DMA1_Channel3->CCR, DMA_CCR3_CIRC);
 //Set peripheral no increment mode
  CLEAR_BIT(DMA1_Channel3->CCR, DMA_CCR3_PINC);
  //Set memory increment mode
  SET_BIT(DMA1_Channel3->CCR, DMA_CCR3_MINC);
  //Set peripheral data width
  MODIFY_REG(DMA1_Channel3->CCR, DMA_CCR3_PSIZE_1, DMA_CCR3_PSIZE_0);
  //Set memory data width
  MODIFY_REG(DMA1_Channel3->CCR, DMA_CCR3_MSIZE_1, DMA_CCR3_MSIZE_0);
  //8 bit 0:0
//  MODIFY_REG(SPI1->CR1, SPI_CR1_BIDIMODE | SPI_CR1_BIDIOE | SPI_CR1_CRCEN | SPI_CR1_CRCNEXT | \
//                        SPI_CR1_DFF | SPI_CR1_RXONLY | SPI_CR1_LSBFIRST | \
//                        SPI_CR1_BR_2 | SPI_CR1_CPOL | SPI_CR1_CPHA | SPI_CR1_SSM | SPI_CR1_SSI | SPI_CR1_MSTR, \
//                        SPI_CR1_BR_1 | SPI_CR1_BR_0);
  //16 bit 0:0
//  MODIFY_REG(SPI1->CR1, SPI_CR1_BIDIMODE | SPI_CR1_BIDIOE | SPI_CR1_CRCEN | SPI_CR1_CRCNEXT | \
//                        SPI_CR1_RXONLY | SPI_CR1_LSBFIRST | SPI_CR1_SSM | SPI_CR1_SSI | SPI_CR1_MSTR, \
//                        SPI_CR1_BR_2 | SPI_CR1_CPOL | SPI_CR1_CPHA , \
//                        SPI_CR1_BR_1 | SPI_CR1_DFF | SPI_CR1_BR_0);
  //16 bit 1:1
  MODIFY_REG(SPI1->CR1, SPI_CR1_BIDIMODE | SPI_CR1_BIDIOE | SPI_CR1_CRCEN | SPI_CR1_CRCNEXT | \
                        SPI_CR1_RXONLY | SPI_CR1_LSBFIRST | SPI_CR1_BR_2 | SPI_CR1_SSM | SPI_CR1_SSI | SPI_CR1_MSTR, \
                        SPI_CR1_BR_1 | SPI_CR1_DFF | SPI_CR1_BR_0 | SPI_CR1_CPOL | SPI_CR1_CPHA);
  CLEAR_BIT(SPI1->CR2, 0x00FF);
  
}
//----------------------------------------------------------
static void SPI2_Init(void)
{
  //SPI2 GPIO
  SET_BIT(RCC->APB2ENR, RCC_APB2ENR_IOPBEN);
  MODIFY_REG(GPIOB->CRH, GPIO_CRH_CNF13_0 | GPIO_CRH_CNF14_1 | GPIO_CRH_CNF15_0 | GPIO_CRH_MODE14, 
                          GPIO_CRH_CNF13_1 | GPIO_CRH_CNF14_0 | GPIO_CRH_CNF15_1 | GPIO_CRH_MODE13 | GPIO_CRH_MODE15);
  //RCC peripheral clock enabling
  SET_BIT(RCC->APB1ENR, RCC_APB1ENR_SPI2EN);
  //Delay after an RCC peripheral clock enabling
  tmpreg = READ_BIT(RCC->APB1ENR, RCC_APB1ENR_SPI2EN);
  //8 bit 0:0
//  MODIFY_REG(SPI2->CR1, SPI_CR1_BIDIMODE | SPI_CR1_BIDIOE | SPI_CR1_CRCEN | SPI_CR1_CRCNEXT | \
//                        SPI_CR1_DFF | SPI_CR1_RXONLY | SPI_CR1_LSBFIRST | \
//                        SPI_CR1_BR_2 | SPI_CR1_CPOL | SPI_CR1_CPHA , \
//                        SPI_CR1_SSM | SPI_CR1_SSI | SPI_CR1_BR_1 | SPI_CR1_BR_0 | SPI_CR1_MSTR);
  //16 bit 0:0
//  MODIFY_REG(SPI2->CR1, SPI_CR1_BIDIMODE | SPI_CR1_BIDIOE | SPI_CR1_CRCEN | SPI_CR1_CRCNEXT | \
//                        SPI_CR1_RXONLY | SPI_CR1_LSBFIRST | \
//                        SPI_CR1_BR_2 | SPI_CR1_CPOL | SPI_CR1_CPHA , \
//                        SPI_CR1_SSM | SPI_CR1_SSI | SPI_CR1_BR_1 | SPI_CR1_DFF | SPI_CR1_BR_0 | SPI_CR1_MSTR);
  //16 bit 1:1
  MODIFY_REG(SPI2->CR1, SPI_CR1_BIDIMODE | SPI_CR1_BIDIOE | SPI_CR1_CRCEN | SPI_CR1_CRCNEXT | \
                        SPI_CR1_RXONLY | SPI_CR1_LSBFIRST | SPI_CR1_BR_2, SPI_CR1_BR_0 | \
                        SPI_CR1_SSM | SPI_CR1_SSI | SPI_CR1_BR_1 | SPI_CR1_DFF | \
                        SPI_CR1_MSTR | SPI_CR1_CPOL | SPI_CR1_CPHA);
  CLEAR_BIT(SPI2->CR2, 0x00FF);
}
//----------------------------------------------------------
static void  DMA1_Init(void)
{
  //DMA controller clock enable
  SET_BIT(RCC->AHBENR, RCC_AHBENR_DMA1EN);
  tmpreg = READ_BIT(RCC->AHBENR, RCC_AHBENR_DMA1EN);
  (void)tmpreg;
  NVIC_EnableIRQ(DMA1_Channel2_IRQn);
  NVIC_EnableIRQ(DMA1_Channel3_IRQn);
}
//----------------------------------------------------------
int main(void)
{
  uint16_t i, n;
  RCC_DeInit();
  SET_BIT(RCC->APB2ENR, RCC_APB2ENR_AFIOEN);
  //Delay after an RCC peripheral clock enabling
  tmpreg = READ_BIT(RCC->APB2ENR, RCC_APB2ENR_AFIOEN);
  //NOJTAG: JTAG-DP Disabled and SW-DP Enabled 
  CLEAR_BIT(AFIO->MAPR,AFIO_MAPR_SWJ_CFG);
  SET_BIT(AFIO->MAPR, AFIO_MAPR_SWJ_CFG_JTAGDISABLE);
  SetSysClockTo72();
  SysTick_Init();
  GPIO_Init();
  DMA1_Init();
  SPI1_Init();
  SPI2_Init();
  delay_ms(100);
  CS2_RESET();
  SPI2_ENABLE();
  Init_7219();
  Clear_7219();
  Number_7219(87654321);
  //fill src buffer
  for(i=0;i<128;i++)
  {
    src_buf[i] = 128 - i;
  }
  //Disable DMA channels
  CLEAR_BIT(DMA1_Channel2->CCR, DMA_CCR2_EN);
  CLEAR_BIT(DMA1_Channel3->CCR, DMA_CCR3_EN);
  //Clear Channel 2 transfer complete flag
  WRITE_REG(DMA1->IFCR, DMA_IFCR_CTCIF2);
  //Clear Channel 2 transfer error flag
  WRITE_REG(DMA1->IFCR, DMA_IFCR_CTEIF2);
  //Clear Channel 3 transfer complete flag
  WRITE_REG(DMA1->IFCR, DMA_IFCR_CTCIF3);
  //Clear Channel 3 transfer error flag
  WRITE_REG(DMA1->IFCR, DMA_IFCR_CTEIF3);
  //Enable DMA Tx SPI1
  SET_BIT(SPI1->CR2, SPI_CR2_TXDMAEN);
  //Enable DMA Rx SPI1
  SET_BIT(SPI1->CR2, SPI_CR2_RXDMAEN);
  //Enable Transfer complete interrupt Channel2
  SET_BIT(DMA1_Channel2->CCR, DMA_CCR2_TCIE);
  //Enable Transfer error interrupt Channel2
  SET_BIT(DMA1_Channel2->CCR, DMA_CCR2_TEIE);
  //Enable Transfer complete interrupt Channel3
  SET_BIT(DMA1_Channel3->CCR, DMA_CCR3_TCIE);
  //Enable Transfer error interrupt Channel3
  SET_BIT(DMA1_Channel3->CCR, DMA_CCR3_TEIE);
  SPI1_ENABLE();  
  while(1)
	{
    //Disable DMA channels
    CLEAR_BIT(DMA1_Channel2->CCR, DMA_CCR2_EN);
    CLEAR_BIT(DMA1_Channel3->CCR, DMA_CCR3_EN);
    //Set Number of data to transfer
    MODIFY_REG(DMA1_Channel2->CNDTR, DMA_CNDTR2_NDT, 128);
    MODIFY_REG(DMA1_Channel3->CNDTR, DMA_CNDTR3_NDT, 128);
    //Configure the Source and Destination addresses
    WRITE_REG(DMA1_Channel2->CPAR, (uint32_t)&(SPI1->DR));
    WRITE_REG(DMA1_Channel2->CMAR, (uint32_t)&dst_buf);
    WRITE_REG(DMA1_Channel3->CPAR, (uint32_t)&(SPI1->DR));
    WRITE_REG(DMA1_Channel3->CMAR, (uint32_t)&src_buf);
    //Enable DMA channels
    SET_BIT(DMA1_Channel3->CCR, DMA_CCR3_EN);
    SET_BIT(DMA1_Channel2->CCR, DMA_CCR2_EN);
    while(!fl) {}
    fl=0;
    for(i=0;i<128;i++)
    {
      src_buf[i] = 8192 - dst_buf[i];
    }
    for(n=0;n<128;n++)
    {
      NumberR_7219(dst_buf[n]);
      NumberL_7219(src_buf[n]);
      delay_ms(50);
    }
  }
}
//----------------------------------------------------------
void SysTick_Handler(void)
{
  if(SysTick_CNT > 0)  SysTick_CNT--;
}
//----------------------------------------------------------
void DMA1_Channel2_IRQHandler(void)
{
  if(READ_BIT(DMA1->ISR, DMA_ISR_TCIF2) == (DMA_ISR_TCIF2))
  {
    //Clear Channel 2 global interrupt flag
    WRITE_REG(DMA1->IFCR, DMA_IFCR_CGIF2);
  }
  else if(READ_BIT(DMA1->ISR, DMA_ISR_TEIF2) == (DMA_ISR_TEIF2))
  {
    __NOP();
  }
}
//----------------------------------------------------------
void DMA1_Channel3_IRQHandler(void)
{
  if(READ_BIT(DMA1->ISR, DMA_ISR_TCIF3) == (DMA_ISR_TCIF3))
  {
    //Clear Channel 3 global interrupt flag
    WRITE_REG(DMA1->IFCR, DMA_IFCR_CGIF3);
    fl = 1;
  }
  else if(READ_BIT(DMA1->ISR, DMA_ISR_TEIF3) == (DMA_ISR_TEIF3))
  {
    __NOP();
  }
}
//----------------------------------------------------------
